import numpy as np
import matplotlib.pyplot as plt
from matplotlib import animation

# Low storage Runge-Kutta coefficients

rk4a = np.array([0.0,\
        -567301805773.0/1357537059087.0,\
        -2404267990393.0/2016746695238.0,\
        -3550918686646.0/2091501179385.0,\
        -1275806237668.0/842570457699.0])
        
rk4b = np.array([1432997174477.0/9575080441755.0,\
        5161836677717.0/13612068292357.0,\
        1720146321549.0/2090206949498.0,\
        3134564353537.0/4481467310338.0,\
        2277821191437.0/14882151754819.0])
         
rk4c = np.array([0.0,\
        1432997174477.0/9575080441755.0,\
        2526269341429.0/6820363962896.0,\
        2006345519317.0/3224310063776.0,\
        2802321613138.0/2924317926251.0])

# Boundary condition
def uBound(t):
        return -np.sin(t)

# Initial condition
def uInit(x):
        return np.sin(x)


# This function should compute the right hand side of the semidiscrete scheme
# Parameters:
#               locJac          Local Jacobian
#               diffMatrix      Differentiation matrix
#               LIFT            Boundary integral operator
#               U               Current time approximation
#               Np              Number of points per element
#               Nel             Number of elements
#               t               Current time
#               a               Advection velocity

def advectRHS(locJac, diffMatrix, LIFT, U, Np, Nel, t, a, alpha):
        
        RHSU  = np.empty( (Np ,Nel) )
	du    = np.empty( (2,Nel) )
	# ...
	# ...
	# ...
        return RHSU


# This function should compute the boundary integral contributing to the
# right hand side of the semidiscrete scheme
# Parameters:
#               x_LGL       Gauss-Lobatto integration points

def surfaceInt( x_LGL ):
	LIFT = np.empty((k,2))
	# ...
	# ...
	# ...
        return LIFT

# Run DG method
# Parameters:
#               Nel         Number of elements
#               degree      Polynomial degree per element

def run_dg(Nel, degree, alpha=1.):

        # Number of points per element
        Np  = degree + 1

        # Advection velocity
        a   = 2 * np.pi

        # Define mesh
        x_init  = 0.
        x_end   = 2 * np.pi
        xp      = np.linspace(x_init, x_end, Nel + 1)
        r       = ...		    # Gauss-Lobatto points in the reference element
        xr      = ...		    # Mapped Gauss-Lobatto points to each element

        # Initialize local operators
        Dr   = ...      	    # Differentiation matrix
        LIFT = surfaceInt(r)        # Boundary integral
        Jx   = np.dot(Dr,xr)        # Local Jacobian

        # Residual storage for Runge-Kutta
        resu = np.zeros((Np,Nel))
	
        # Set up time-related parameters
        t_init = 0.
        t_end  = 10.
        time  = [t_init,]
        CFL    = .75
        dt     = .5 * ( CFL / a ) * np.min(np.diff(xr,axis=0))
        Nstp   = np.int( np.ceil(t_end / dt) )
        dt     = t_end / Nstp
	
	# set initial condition
        Uh   = [uInit(xr),]
	
	# Integrate in time with low storage Runge-Kutta
        for tstep in range(0,Nstp):
                for irk in range(0,5):
                        t_loc = time[-1] + rk4c[irk] * dt
                        rhsu  = advectRHS(Jx, Dr, LIFT, Uh[-1], Np, Nel,\
					t_loc, a, alpha)
                        resu  = resu * rk4a[irk] + rhsu * dt
                        Uh.append( np.array(Uh[-1]  + resu * rk4b[irk] ) )
                time.append(time[-1] + dt)

	return xr, time, Uh


# ################ BEGIN: Animation code ################ #

# First set up the figure, the axis, and the plot element we want to animate
def set_up(Nel,degree):
    global fig, ax, lines, time_text
    fig = plt.figure()
    ax  = plt.axes(ylim=(-1.5, 1.5), xlim=(0, 2*np.pi))
    ax.set_ylabel('$u(x,t)$')
    ax.set_xlabel('$x$')
    ax.set_title('DG approx. with %i elements of degree %i' %(Nel,degree))
    time_text = ax.text(0.02, 0.95, '', transform=ax.transAxes)
    lines = []
    for el in range(0,Nel):
        lobj = ax.plot([],[])[0]
	lines.append ( lobj )

# initialization function: plot the background of each frame
def init():
    time_text.set_text('')
    for line in lines:
        line.set_data([],[])
    return tuple(lines) + (time_text,)

# animation function.  This is called sequentially
def animate(j):
    time_text.set_text('Time = %.2f' % t[j])
    el = 0
    for line in lines: 
	    line.set_data(x[:,el], U[j][:,el])
            el += 1
    return tuple(lines) + (time_text,)
 
def run(Nel,degree):
    global x,t,U
    set_up(Nel,degree)
    x, t, U = run_dg(Nel,degree)
    anim = animation.FuncAnimation(fig, animate, init_func=init,
                               frames=np.size(t), interval=80, blit=True)
    plt.show()

# ################ END: Animation code ################ #
